@@ -2,7 +2,7 @@
     MagicHash, Rank2Types, ScopedTypeVariables, TypeFamilies, UnboxedTuples #-}
 -- |
 -- Module    : System.Random.MWC
--- Copyright : (c) 2009, 2010, 2011 Bryan O'Sullivan
+-- Copyright : (c) 2009-2012 Bryan O'Sullivan
 -- License   : BSD3
 --
 -- Maintainer  : bos@serpentine.com
@@ -10,34 +10,75 @@
 -- Portability : portable
 --
 -- Pseudo-random number generation.  This module contains code for
--- generating high quality random numbers that follow either a uniform
--- or normal distribution.
+-- generating high quality random numbers that follow a uniform
+-- distribution.
+--
+-- For non-uniform distributions, see the
+-- 'System.Random.MWC.Distributions' module.
 --
 -- The uniform PRNG uses Marsaglia's MWC256 (also known as MWC8222)
 -- multiply-with-carry generator, which has a period of 2^8222 and
 -- fares well in tests of randomness.  It is also extremely fast,
 -- between 2 and 3 times faster than the Mersenne Twister.
+--
+-- The generator state is stored in the 'Gen' data type. It can be
+-- created in several ways:
+--
+--   1. Using the 'withSystemRandom' call, which creates a random state.
+--
+--   2. Supply your own seed to 'initialize' function.
+--
+--   3. Finally, 'create' makes a generator from a fixed seed.
+--      Generators created in this way aren't really random.
+--
+-- For repeatability, the state of the generator can be snapshotted
+-- and replayed using the 'save' and 'restore' functions.
+--
+-- The simplest use is to generate a vector of uniformly distributed values:
+--
+-- @
+--   vs <- withSystemRandom (uniformVector 100)
+-- @
+--
+-- These values can be of any type which is an instance of the class 'Variate'.
+--
+-- To generate random values on demand, first 'create' a random number generator.
+--
+-- @
+--   gen <- create
+-- @
+--
+-- Keep this generator and use it wherever random values are required. Get a random
+-- value using 'uniform' or 'uniformR':
+--
+-- @
+--   v <- uniform gen
+-- @
+--
+-- @
+--   v <- uniformR (1, 52) gen
+-- @
 module System.Random.MWC
     (
-    -- * Types
+    -- * Gen: Pseudo-Random Number Generators
       Gen
     , GenIO
     , GenST
-    , Seed
-    , fromSeed
-    , toSeed
-    , Variate(..)
-    -- * Other distributions
-    , normal
-    -- * Creation
     , create
     , initialize
     , withSystemRandom
-    -- * State management
+
+    -- * Variates: uniformly distributed values
+    , Variate(..)
+    , uniformVector
+
+    -- * Seed: state management
+    , Seed
+    , fromSeed
+    , toSeed
     , save
     , restore
-    -- * Helper functions
-    , uniformVector
+
     -- * References
     -- $references
     ) where
@@ -56,23 +97,19 @@ import Data.IORef              (atomicModifyIORef, newIORef)
 import Data.Ratio              ((%), numerator)
 import Data.Time.Clock.POSIX   (getPOSIXTime)
 import Data.Typeable           (Typeable)
-import Data.Vector.Generic     (Vector, unsafeFreeze)
+import Data.Vector.Generic     (Vector)
 import Data.Word               (Word, Word8, Word16, Word32, Word64)
 import Foreign.Marshal.Alloc   (allocaBytes)
 import Foreign.Marshal.Array   (peekArray)
 import Prelude hiding (catch)
 import qualified Data.Vector.Generic         as G
-import qualified Data.Vector.Generic.Mutable as GM
 import qualified Data.Vector.Unboxed         as I
 import qualified Data.Vector.Unboxed.Mutable as M
 import System.CPUTime   (cpuTimePrecision, getCPUTime)
 import System.IO        (IOMode(..), hGetBuf, hPutStrLn, stderr, withBinaryFile)
 import System.IO.Unsafe (unsafePerformIO)
 
--- FIXME: removal of Unbox constraint leads to severe (~10x)
---        performance drop with GHC 6.12. For details see bug #33 in the 
---        vector bug tracker[1]
--- [1] http://trac.haskell.org/vector/ticket/33
+
 
 -- | The class of types for which we can generate uniformly
 -- distributed random variates.
@@ -84,7 +121,7 @@ import System.IO.Unsafe (unsafePerformIO)
 --
 -- /Note/: Marsaglia's PRNG is not known to be cryptographically
 -- secure, so you should not use it for cryptographic operations.
-class M.Unbox a => Variate a where
+class Variate a where
     -- | Generate a single uniformly distributed random variate.  The
     -- range of values produced varies by type:
     --
@@ -111,49 +148,49 @@ class M.Unbox a => Variate a where
 
 instance Variate Int8 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Int16 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Int32 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Int64 where
     uniform  = uniform2 wordsTo64Bit
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Word8 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Word16 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Word32 where
     uniform  = uniform1 fromIntegral
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Word64 where
     uniform  = uniform2 wordsTo64Bit
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
@@ -184,7 +221,7 @@ instance Variate Int where
 #else
     uniform = uniform2 wordsTo64Bit
 #endif
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
@@ -194,7 +231,7 @@ instance Variate Word where
 #else
     uniform = uniform2 wordsTo64Bit
 #endif
-    uniformR = uniformRange
+    uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
@@ -287,10 +324,12 @@ create = initialize defaultSeed
 -- verbatim, then its elements are 'xor'ed against elements of the
 -- default seed until 256 elements are reached.
 --
--- If a seed contains exactly 258 elements last two elements are used
--- to set generator state. It's to ensure that @gen' == gen@
+-- If a seed contains exactly 258 elements, then the last two elements
+-- are used to set the generator's initial state. This allows for
+-- complete generator reproducibility, so that e.g. @gen' == gen@ in
+-- the following example:
 --
--- > gen' <- initialize . fromSeed =<< save
+-- @gen' <- 'initialize' . 'fromSeed' =<< 'save'@
 initialize :: (PrimMonad m, Vector v Word32) =>
               v Word32 -> m (Gen (PrimState m))
 initialize seed = do
@@ -330,29 +369,14 @@ newtype Seed = Seed {
 toSeed :: (Vector v Word32) => v Word32 -> Seed
 toSeed v = Seed $ I.create $ do { Gen q <- initialize v; return q }
 
--- Safe version of unsafeFreeze.
--- NOTE: vector-0.7 will provide function `freeze' with same
---       functionality. This function shall be removed when support for
---       vector<=0.6 is dropped
-safeFreeze :: (PrimMonad m, Vector v a) => G.Mutable v (PrimState m) a -> m (v a)
-safeFreeze v = do
-  v' <- GM.unsafeNew (GM.length v)
-  GM.unsafeCopy v' v
-  unsafeFreeze v'
-
 -- | Save the state of a 'Gen', for later use by 'restore'.
 save :: PrimMonad m => Gen (PrimState m) -> m Seed
-save (Gen q) = Seed `liftM` safeFreeze q
+save (Gen q) = Seed `liftM` G.freeze q
 {-# INLINE save #-}
 
--- NOTE: with vector-0.7 all code could be replaced with `clone'
 -- | Create a new 'Gen' that mirrors the state of a saved 'Seed'.
 restore :: PrimMonad m => Seed -> m (Gen (PrimState m))
-restore (Seed s) = M.unsafeNew n >>= fill
-  where fill q = go 0 where
-          go !i | i >= n    = return $! Gen q
-                | otherwise = M.unsafeWrite q i (I.unsafeIndex s i) >> go (i+1)
-        n = I.length s
+restore (Seed s) = Gen `liftM` G.thaw s
 {-# INLINE restore #-}
 
 
@@ -379,6 +403,10 @@ acquireSeedSystem = do
 -- pseudo-random numbers (\"\/dev\/urandom\" on Unix-like systems),
 -- then run the given action.
 --
+-- This is a heavyweight function, intended to be called only
+-- occasionally (e.g. once per thread).  You should use the `Gen` it
+-- creates to generate many random numbers.
+--
 -- /Note/: on Windows, this code does not yet use the native
 -- Cryptographic API as a source of random numbers (it uses the system
 -- clock instead). As a result, the sequences it generates may not be
@@ -404,16 +432,16 @@ nextIndex i = fromIntegral j
     where j = fromIntegral (i+1) :: Word8
 {-# INLINE nextIndex #-}
 
-a :: Word64
-a = 1540315826
-{-# INLINE a #-}
+aa :: Word64
+aa = 1540315826
+{-# INLINE aa #-}
 
 uniformWord32 :: PrimMonad m => Gen (PrimState m) -> m Word32
 uniformWord32 (Gen q) = do
   i  <- nextIndex `liftM` M.unsafeRead q ioff
   c  <- fromIntegral `liftM` M.unsafeRead q coff
   qi <- fromIntegral `liftM` M.unsafeRead q i
-  let t  = a * qi + c
+  let t  = aa * qi + c
       c' = fromIntegral (t `shiftR` 32)
       x  = fromIntegral t + c'
       (# x', c'' #)  | x < c'    = (# x + 1, c' + 1 #)
@@ -437,12 +465,12 @@ uniform2 f (Gen q) = do
   c  <- fromIntegral `liftM` M.unsafeRead q coff
   qi <- fromIntegral `liftM` M.unsafeRead q i
   qj <- fromIntegral `liftM` M.unsafeRead q j
-  let t   = a * qi + c
+  let t   = aa * qi + c
       c'  = fromIntegral (t `shiftR` 32)
       x   = fromIntegral t + c'
       (# x', c'' #)  | x < c'    = (# x + 1, c' + 1 #)
                      | otherwise = (# x,     c' #)
-      u   = a * qj + fromIntegral c''
+      u   = aa * qj + fromIntegral c''
       d'  = fromIntegral (u `shiftR` 32)
       y   = fromIntegral u + d'
       (# y', d'' #)  | y < d'    = (# y + 1, d' + 1 #)
@@ -475,31 +503,36 @@ type instance Unsigned Word   = Word
 -- unsigned data type of same size
 sub :: (Integral a, Integral (Unsigned a)) => a -> a -> Unsigned a
 sub x y = fromIntegral x - fromIntegral y
+{-# INLINE sub #-}
 
 add :: (Integral a, Integral (Unsigned a)) => a -> Unsigned a -> a
 add m x = m + fromIntegral x
+{-# INLINE add #-}
 
--- Generate uniform value in the range [0,n). Values must be
--- unsigned. Second parameter is random number generator
-unsignedRange :: (PrimMonad m, Integral a, Bounded a) => a -> m a -> m a
-unsignedRange n rnd = go
-  where
-    buckets = maxBound `div` n
-    maxN    = buckets * n
-    go = do x <- rnd
-            if x < maxN then return (x `div` buckets)
-                        else go
-{-# INLINE unsignedRange #-}
-
--- Generate unformly distributed value in inclusive range.
+-- Generate uniformly distributed value in inclusive range.
+--
+-- NOTE: This function must be fully applied. Otherwise it won't be
+--       inlined, which will cause a severe performance loss.
+--
+-- > uniformR     = uniformRange      -- won't be inlined
+-- > uniformR a b = uniformRange a b  -- will be inlined
 uniformRange :: ( PrimMonad m
                 , Integral a, Bounded a, Variate a
                 , Integral (Unsigned a), Bounded (Unsigned a), Variate (Unsigned a))
              => (a,a) -> Gen (PrimState m) -> m a
 uniformRange (x1,x2) g
-  | x1 == minBound && x2 == maxBound = uniform g
-  | otherwise                        = do x <- unsignedRange (sub x2 x1 + 1) (uniform g)
-                                          return $! add x1 x
+  | n == 0    = uniform g   -- Abuse overflow in unsigned types
+  | otherwise = loop
+  where
+    -- Allow ranges where x2<x1
+    (# i, j #) | x1 < x2   = (# x1, x2 #)
+               | otherwise = (# x2, x1 #)
+    n       = 1 + sub j i
+    buckets = maxBound `div` n
+    maxN    = buckets * n
+    loop    = do x <- uniform g
+                 if x < maxN then return $! add i (x `div` buckets)
+                             else loop
 {-# INLINE uniformRange #-}
 
 -- | Generate a vector of pseudo-random variates.  This is not
@@ -510,56 +543,6 @@ uniformVector :: (PrimMonad m, Variate a, Vector v a)
 uniformVector gen n = G.replicateM n (uniform gen)
 {-# INLINE uniformVector #-}
 
-data T = T {-# UNPACK #-} !Double {-# UNPACK #-} !Double
-
--- | Generate a normally distributed random variate.
---
--- The implementation uses Doornik's modified ziggurat algorithm.
--- Compared to the ziggurat algorithm usually used, this is slower,
--- but generates more independent variates that pass stringent tests
--- of randomness.
-normal :: PrimMonad m => Gen (PrimState m) -> m Double
-normal gen = loop
-  where
-    loop = do
-      u  <- (subtract 1 . (*2)) `liftM` uniform gen
-      ri <- uniform gen
-      let i  = fromIntegral ((ri :: Word32) .&. 127)
-          bi = I.unsafeIndex blocks i
-          bj = I.unsafeIndex blocks (i+1)
-      if abs u < I.unsafeIndex ratios i
-        then return $! u * bi
-        else if i == 0
-        then normalTail (u < 0)
-        else do
-          let x  = u * bi
-              xx = x * x
-              d  = exp (-0.5 * (bi * bi - xx))
-              e  = exp (-0.5 * (bj * bj - xx))
-          c <- uniform gen
-          if e + c * (d - e) < 1
-            then return x
-            else loop
-    blocks = let f = exp (-0.5 * r * r)
-             in (`I.snoc` 0) . I.cons (v/f) . I.cons r .
-                I.unfoldrN 126 go $! T r f
-      where
-        go (T b g)   = let !u = T h (exp (-0.5 * h * h))
-                           h  = sqrt (-2 * log (v / b + g))
-                       in Just (h, u)
-        v            = 9.91256303526217e-3
-    {-# NOINLINE blocks #-}
-    r                = 3.442619855899
-    ratios           = I.zipWith (/) (I.tail blocks) blocks
-    {-# NOINLINE ratios #-}
-    normalTail neg  = tailing
-      where tailing  = do
-              x <- ((/r) . log) `liftM` uniform gen
-              y <- log          `liftM` uniform gen
-              if y * (-2) < x * x
-                then tailing
-                else return $! if neg then x - r else r - x
-{-# INLINE normal #-}
 
 defaultSeed :: I.Vector Word32
 defaultSeed = I.fromList [
@@ -610,15 +593,6 @@ defaultSeed = I.fromList [
 
 -- $references
 --
--- * Doornik, J.A. (2005) An improved ziggurat method to generate
---   normal random samples. Mimeo, Nuffield College, University of
---   Oxford.  <http://www.doornik.com/research/ziggurat.pdf>
---
--- * Doornik, J.A. (2007) Conversion of high-period random numbers to
---   floating point.
---   /ACM Transactions on Modeling and Computer Simulation/ 17(1).
---   <http://www.doornik.com/research/randomdouble.pdf>
---
 -- * Marsaglia, G. (2003) Seeds for random number generators.
 --   /Communications of the ACM/ 46(5):90&#8211;93.
 --   <http://doi.acm.org/10.1145/769800.769827>
